<?php

/**
 * Componente para acceder desde un esquema a objetos de otros esquemas en Oracle
 *
 * @author Alvaro Muñoz
 * @link https://dosmweb.com/blog/
 */

namespace app\components;

class OciMultiSchema extends \yii\db\oci\Schema {
	protected function findColumns($table) {
		$sql = <<<SQL
SELECT
	A.COLUMN_NAME,
	A.DATA_TYPE,
	A.DATA_PRECISION,
	A.DATA_SCALE,
	A.DATA_LENGTH,
	A.NULLABLE,
	A.DATA_DEFAULT,
	COM.COMMENTS AS COLUMN_COMMENT
FROM ALL_TAB_COLUMNS A
	INNER JOIN ALL_OBJECTS B ON B.OWNER = A.OWNER AND LTRIM(B.OBJECT_NAME) = LTRIM(A.TABLE_NAME)
	LEFT JOIN ALL_COL_COMMENTS COM ON (A.OWNER = COM.OWNER AND A.TABLE_NAME = COM.TABLE_NAME AND A.COLUMN_NAME = COM.COLUMN_NAME)
WHERE
	A.OWNER = :schemaName
	AND B.OBJECT_TYPE IN ('TABLE', 'VIEW', 'MATERIALIZED VIEW', 'SYNONYM')
	AND B.OBJECT_NAME = :tableName
ORDER BY A.COLUMN_ID
SQL;
		try {
			$columns = $this->db->createCommand($sql, [
				':tableName' => $table->name,
				':schemaName' => $table->schemaName,
			])->queryAll();
			if(isset(\Yii::$app->params['esquemas'][$table->name])){
				$owner = \Yii::$app->params['esquemas'][$table->name];
				$sql = <<<SQL
SELECT
	A.COLUMN_NAME,
	A.DATA_TYPE,
	A.DATA_PRECISION,
	A.DATA_SCALE,
	A.DATA_LENGTH,
	A.NULLABLE,
	A.DATA_DEFAULT,
	COM.COMMENTS AS COLUMN_COMMENT
FROM ALL_TAB_COLUMNS A
	INNER JOIN ALL_OBJECTS B ON B.OWNER = A.OWNER AND LTRIM(B.OBJECT_NAME) = LTRIM(A.TABLE_NAME)
	LEFT JOIN ALL_COL_COMMENTS COM ON (A.OWNER = COM.OWNER AND A.TABLE_NAME = COM.TABLE_NAME AND A.COLUMN_NAME = COM.COLUMN_NAME)
WHERE  B.OBJECT_TYPE IN ('TABLE', 'VIEW', 'MATERIALIZED VIEW', 'SYNONYM')
	AND B.OBJECT_NAME = :tableName
	AND A.OWNER = :schemaName
ORDER BY A.COLUMN_ID
SQL;
				$columns2 = $this->db->createCommand($sql, [
					':tableName' => $table->name,
					':schemaName' => $owner,
				])->queryAll();
				$columns = $columns + $columns2;
			}
		} catch (\Exception $e) {
			return false;
		}

		if (empty($columns)) {
			return false;
		}

		foreach ($columns as $column) {
			$c = $this->createColumn($column);
			$table->columns[$c->name] = $c;
			if ($c->isPrimaryKey) {
				$table->primaryKey[] = $c->name;
				$table->sequenceName = $this->getTableSequenceName($table->name);
				if (!empty($table->sequenceName))
					$c->autoIncrement = true;
			}
		}
		return true;
	}

		protected function findSchemaNames()
	{
		$sql = <<<SQL
SELECT
	USERNAME
FROM DBA_USERS U
WHERE
	EXISTS (SELECT 1 FROM DBA_OBJECTS O WHERE O.OWNER = U.USERNAME)
	AND DEFAULT_TABLESPACE NOT IN ('SYSTEM','SYSAUX')
SQL;
		return $this->db->createCommand($sql)->queryColumn();
	}

	/**
	 * @inheritdoc
	 */
	protected function findTableNames($schema = '') {
		if ($schema === '') {
			$sql = <<<SQL
SELECT
	TABLE_NAME
FROM USER_TABLES
UNION ALL
SELECT
	VIEW_NAME AS TABLE_NAME
FROM USER_VIEWS
UNION ALL
SELECT
	MVIEW_NAME AS TABLE_NAME
FROM USER_MVIEWS
UNION ALL
select SYNONYM_NAME TABLE_NAME from USER_SYNONYMS
ORDER BY TABLE_NAME
SQL;
			$command = $this->db->createCommand($sql);
		} else {
			$sql = <<<SQL
SELECT
	OBJECT_NAME AS TABLE_NAME
FROM ALL_OBJECTS
WHERE
	OBJECT_TYPE IN ('TABLE', 'VIEW', 'MATERIALIZED VIEW', 'SYNONYM')
	AND OWNER = :schema
ORDER BY OBJECT_NAME
SQL;
			$command = $this->db->createCommand($sql, [':schema' => $schema]);
		}

		$rows = $command->queryAll();
		$names = [];
		foreach ($rows as $row) {
			if ($this->db->slavePdo->getAttribute(\PDO::ATTR_CASE) === \PDO::CASE_LOWER) {
				$row = array_change_key_case($row, CASE_UPPER);
			}
			$names[] = $row['TABLE_NAME'];
		}
		return $names;
	}

}